from django.conf import settings
from django.db import models
from django.db.models import permalink

from django.dispatch import dispatcher
from django.db.models import signals
from django.core.cache import cache

from django.utils.translation import ugettext_lazy as _

from django.contrib.sites.models import Site
from django.contrib.auth.models import User, Message

from datetime import datetime


class Tag(models.Model) :
    name = models.CharField(_(u'Tag Name'), max_length=25, unique=True, blank=False)
    create_date = models.DateField(_(u'Create date'), default=datetime.now, editable=False)

    def __unicode__(self) :
        return self.name

    def get_absolute_url(self) :
        """
        Return Entry list URL tagged by this Tag.
        """
        return ('tag_list', (), { 'tag': self.name })
    get_absolute_url = permalink(get_absolute_url)

    class Meta:
        ordering = (u'name',)
        verbose_name = _(u'Tag')
        verbose_name_plural = _(u'Tags')

    class Admin :
        list_display = ('name',)

class EntryPublicManager(models.Manager) :
    def get_query_set(self) :
        return super(EntryPublicManager, self).get_query_set().filter(visible=True).filter(pub_date__lte=datetime.now)

class Entry(models.Model) :
    title = models.CharField(_(u'Title'), max_length=250, blank=False)
    author = models.ForeignKey(User, verbose_name=_(u'Author'))
    slug = models.SlugField(_(u'Slug'), unique_for_date='pub_date', prepopulate_from=("title",), help_text=_(u'Use for url. You hova to input yourself if your title is not Ascii Charcters.'))
    body = models.TextField(_(u'Body'), blank=False)
    pub_date = models.DateTimeField(_(u'Public from'), default=datetime.now, help_text=_(u'Entry publish date and time.'))
    create_date = models.DateTimeField(_(u'Create date'), editable=False)
    update_date = models.DateTimeField(_(u'Update date'), editable=False)
    visible = models.BooleanField(default=True, help_text=_(u'Unchek if showing this entry for only logined user.'))
    ping_now = models.BooleanField(default=False, help_text=_(u"Ping entry info for ping sites. Attn, it doesn't work if Entry is not visible."))

    trackback_now = models.BooleanField(default=False, help_text=_(u"Ping trackback for tracback urls. Attn, it doesn't work if Entry is not visible."))
    trackbacked_sites = models.TextField(_(u'Trackback urls'), blank=True)

    tags = models.ManyToManyField(Tag, verbose_name=_(u'list of tags'), blank=True)

    comment_count = models.IntegerField(_(u'Comment count'), default=0, editable=False)
    trackback_count = models.IntegerField(_(u'Trackback count'), default=0, editable=False)

    riddle = models.CharField(_(u'Riddle'), max_length=250, db_index=False, blank=True, help_text=_(u'Riddle for commenting this entry.'))
    answer = models.CharField(_(u'Answer'), max_length=50, db_index=False, blank=True, help_text=_(u"Riddle's answer. Ignore case."))

    objects = models.Manager()
    public_objects = EntryPublicManager()


    class Meta:
        ordering = ['-pub_date']
        verbose_name = _(u'Entry')
        verbose_name_plural = _(u'Entries')

    class Admin :
        list_display = ('title', 'pub_date', 'author', 'comment_count', 'trackback_count',)
        list_filter = ('pub_date','visible','tags',)
        date_hierarchy = 'pub_date'
        search_fields = ('title', 'body', 'author',)
        fields = (
            (None, {
                'fields': (('title', 'author',), 'pub_date', 'slug', 'tags', 'visible', 'body', 'ping_now', )
            }),
            (_('Comment options'), {
                'classes': 'collapse',
                'fields' : ('riddle', 'answer',)
            }),
            (_('Trackback options'), {
                'classes': 'collapse',
                'fields' : ('trackback_now', 'trackbacked_sites',)
            }),
        )

    def __unicode__(self) :
        return self.title

    def has_3lines(self) :
        """
        True if body has 3lines or more
        """
        return ( len(self.body.rstrip().split('\n')) > 2 )

    def trackback_array(self) :
        """
        Return this entry's Trackback as list sorted by pub_date.
        """
        return self.trackback_set.order_by('create_date')

    def comment_array(self) :
        """
        Return this entry's Comment as list sorted by pub_date.
        """
        return self.comment_set.order_by('create_date')

    def description(self):
        """
        Return this entry's short summay.
        """
        return self.body[:20]
    description = property(description)

    def get_absolute_url(self):
        """
        Return this Entry's Absolute URL.
        """
        return ('entry_detail', (), { 'year': self.pub_date.year, 'month': self.pub_date.month, 'day': self.pub_date.day, 'slug': self.slug })
    get_absolute_url = permalink(get_absolute_url)

    def month_url(self) :
        """
        Return this entry created month's URL.
        """
        return ('entry_archive_month', (), { 'year': self.pub_date.year, 'month': self.pub_date.month })
    month_url = permalink(month_url)

    def day_url(self) :
        return ('entry_archive_day', (), { 'year': self.pub_date.year, 'month': self.pub_date.month, 'day': self.pub_date.day })
    day_url = permalink(day_url)

    def add_comment_url(self):
        return ('add_comment', (), { 'year': self.pub_date.year, 'month': self.pub_date.month, 'day': self.pub_date.day, 'slug': self.slug })
    add_comment_url = permalink(add_comment_url)

    def tbping_url(self):
        return ('tbping', (), { 'year': self.pub_date.year, 'month': self.pub_date.month, 'day': self.pub_date.day, 'slug': self.slug })
    tbping_url = permalink(tbping_url)

    def is_public(self):
	    return self.visible and (self.pub_date > datetime.now())
    is_public = property(is_public)

    def save(self):
        if not self.id:
            self.create_date = datetime.now()
        self.update_date = datetime.now()
        
        self.should_tb = self.trackback_now and self.is_public and len(self.trackbacked_sites.strip()) > 0
        self.should_ping = self.ping_now and self.is_public
        self.trackback_now = False
        self.ping_now = False
        super(Entry, self).save()
        #clear cache
        if self.should_tb :
            import urllib
            params = urllib.urlencode({'title': self.title, 'url': self.absolute_url, 'excerpt': self.body, 'blog_name': settings.configure})
            pass #TODO implements it.
        if self.should_ping :
            import xmlrpclib
            ping_server_list = Pingsites.objects.all()
            site = Site.objects.get(pk=settings.SITE_ID)
            for ping_server in ping_server_list :
                rpc = xmlrpclib.ServerProxy(ping_server.ping_url)
                xmlrpclib.Transport.user_agent = 'djblogkit'
                try:
                    ret = rpc.weblogUpdates.ping(self.title, 'http://%s%s' % (site.domain, self.absolute_url))
                    msg = '%s: %s' %s (site.domain, ret.get('message', ''))
                except Exception, e:
                    msg = 'FAIL %s: %s' % (site.domain, e)
                Message.objects.create(user=self.author, message=msg)

class Comment(models.Model) :
    entry = models.ForeignKey(Entry)
    author = models.CharField(_(u'Commentator'), max_length=50)
    url = models.URLField(_(u'URL'), blank=True)
    body = models.TextField(_(u'Body'), blank=False)
    create_date = models.DateTimeField(_(u'Comment date'), editable=False)

    class Meta:
        ordering = ('-create_date',)
        verbose_name = _(u'Comment')
        verbose_name_plural = _(u'Comments')

    class Admin :
        list_display = ('create_date', 'author')

    def save(self):
        if not self.id:
            self.create_date = datetime.now()
            e = self.entry
            e.comment_count = e.comment_count + 1
            e.save()
        super(Comment, self).save()

    def delete(self):
        entry = self.entry
        entry.comment_count -= 1
        entry.save()
        super(Comment, self).delete()

class Trackback(models.Model) :
    entry = models.ForeignKey(Entry)
    blog_name = models.CharField(_(u'Blog name'), max_length=200, blank=True)
    url = models.URLField(_(u'URL'), blank=False, verify_exists=False)
    excerpt = models.TextField(_(u'Excerpt'), blank=False)
    create_date = models.DateTimeField(_(u'Trackback date'), blank=True, default=datetime.now)

    class Meta:
        ordering = ('-create_date',)
        verbose_name = _(u'Trackback')
        verbose_name_plural = _(u'Trackbacks')

    class Admin :
        list_display = ('create_date', 'blog_name')

    def save(self):
        if not self.id:
            self.create_date = datetime.now()
            e = self.entry
            e.trackback_count = e.trackback_count + 1
            e.save()
        super(Trackback, self).save()

    def delete(self):
        entry = self.entry
        entry.trackback_count -= 1
        entry.save()
        super(Trackback, self).delete()

class Pingsites(models.Model) :
    name = models.CharField(_(u'name'), max_length=50, blank=False)
    ping_url = models.URLField(_(u'URL'), blank=False, verify_exists=False)

    class Meta:
        ordering = ('id',)
        verbose_name = _(u'Ping site')
        verbose_name_plural = _(u'Ping sites')

    class Admin :
        list_display = ('name',)

def delete_cache():
    if settings.CACHE_BACKEND:
        cache.delete(settings.BLOG_CONTEXT_CACHE_KEY)


#delete blog_context from cache
dispatcher.connect(delete_cache,
    signal=signals.post_save, sender=Tag)
dispatcher.connect(delete_cache,
    signal=signals.post_save, sender=Entry)
dispatcher.connect(delete_cache,
    signal=signals.post_save, sender=Comment)
dispatcher.connect(delete_cache,
    signal=signals.post_save, sender=Trackback)
dispatcher.connect(delete_cache,
    signal=signals.post_delete, sender=Tag)
dispatcher.connect(delete_cache,
    signal=signals.post_delete, sender=Entry)
dispatcher.connect(delete_cache,
    signal=signals.post_delete, sender=Comment)
dispatcher.connect(delete_cache,
    signal=signals.post_delete, sender=Trackback)
