# -*- coding: utf-8 -*-
from django.test import TestCase
from django.conf import settings
from django.core.urlresolvers import reverse

from django.contrib.auth.models import User
from djblogkit.blog.models import Entry

from django.test.client import Client

from datetime import datetime

class EntryTestCase(TestCase):
    fixtures = ['user.json']
    
    def setUp(self):
        current_date = datetime.now()
        self.staff = User.objects.get(username='staff_user')
        Entry.objects.create(title='test', author=self.staff, pub_date=current_date, slug='test', body='test', visible=True)
        Entry.objects.create(title='draft', author=self.staff, pub_date=current_date, slug='draft', body='draft', visible=False)

    def testVisible(self):
        """
        visibleがTrueなエントリはただリクエストすれば正常レスポンスが返る
        """
        e = Entry.objects.get(slug='test')
        response = self.client.get(e.get_absolute_url())
        self.assertEqual(200, response.status_code)
    
    def testNotVisibleForAnonymous(self):
        """
        visibleがFalseなエントリにログインしていないリクエストをすると404レスポンスが返る
        """
        e = Entry.objects.get(slug='draft')
        
        response = self.client.get(e.get_absolute_url())
        self.assertEqual(404, response.status_code)
    
    def testNotVisibleForStaff(self):
        """
        visibleがFalseなエントリにスタッフ権限のあるユーザでログインし、リクエストをすると正常レスポンスが返る。
        ログインしていなければならない機能がないにもかかわらず、ログインユーザによって動作が変わるため、ダミーのビューを用意した。
        """
        e = Entry.objects.get(slug='draft')

        self.assertEqual(404, self.client.get(e.get_absolute_url()).status_code)
        
        self.client.login(username='staff_user', password='staff')

        self.assertEqual(200, self.client.get(e.get_absolute_url()).status_code)
        
    def testNotVisibleForNonStaff(self):
        """
        visibleがFalseなエントリにスタッフ権限の無いユーザでログインし、リクエストをすると404レスポンスが返る。
        ログインしていなければならない機能がないにもかかわらず、ログインユーザによって動作が変わるため、ダミーのビューを用意した。
        """
        e = Entry.objects.get(slug='draft')
        
        self.client.login(username='non_staff', password='non')
        
        self.assertEqual(404, self.client.get(e.get_absolute_url()).status_code)
