# -*- coding: utf-8 -*-
from django import forms
from django.conf import settings
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.core import validators
from django.template.context import RequestContext
from django.shortcuts import render_to_response, get_object_or_404

from django.contrib.syndication.views import feed
from django.views.generic.list_detail import object_list
from django.views.generic.date_based import object_detail

from djblogkit.blog.models import Entry, Comment, Trackback
from djblogkit.blog.urls import tag_feeds

from datetime import datetime
from datetime import date as ddate
from datetime import time as dtime
import time

comment_add_form = forms.FormWrapper(Comment.AddManipulator(), {}, {})

def tag_feed(request, url, tag) :
    """
    特定タグRSSビュー
    
    urlにはrssという文字を渡してください。
    
    tag引数で絞り込んだRSSを出力します。
    """
    tag_feeds[url].tag = tag
    return feed(request, url, feed_dict=tag_feeds)

def tag_list(request, tag) :
    """
    タグ別エントリビュー
    
    tag引数で絞り込んだエントリ一覧を出力します。
    
    レンダリングに利用されるテンプレート blog/tagged_entry_list.html
    
    選択されたタグ名にアクセスするには、 selected_tag_name 変数を使います。
    """
    return object_list(request, queryset=Entry.public_objects.filter(tags__name__iexact=tag).select_related(),
                       paginate_by=settings.NUM_IN_PAGE, allow_empty=True,
                       extra_context=dict(selected_tag_name=tag))

def entry_detail(request, year, month, day, slug):
    """
    エントリ詳細
    
    年月日とスラグでエントリを一つに絞り込んで出力します。
    
    表示フラグがFalseの場合は、スタッフ権限を持ったユーザでログインしていないと表示しません(404を返します)。
    
    レンダリングに利用されるテンプレート blog/entry_detail.html
    """
    if request.user.is_authenticated() and request.user.is_staff:
        query_set = Entry.objects.all()
    else:
        query_set = Entry.public_objects.all()
    return object_detail(request, year=year, month=month, day=day, slug=slug,
                         queryset=query_set.select_related(), date_field='pub_date',
                         slug_field='slug', month_format='%m', allow_future=True, 
                         extra_context=dict(form=comment_add_form), )

def add_comment(request, year, month, day, slug) :
    """
    コメント追加用ビュー
    
    コメントを追加しようとしたエントリに、Riddleが設定されたいた場合には、valid_answerに正しい答えが入っていないとならない。
    
    レンダリングに利用されるテンプレート blog/entry_detail.html
    
    入力フォームは、formという名前で渡されます。
    """
    if request.method == 'GET' :
        print 'get?'
        raise Http404

    manipulator = Comment.AddManipulator()
    entry = _getEntry(year, month, day, slug)
    if request.method == 'POST' :
        new_data = request.POST.copy()
        new_data.update({'entry': entry.id, 'valid_answer': entry.answer})
        if entry.riddle :
            manipulator.fields += [forms.TextField(_(u'answer'), max_length=50, is_required=True, validator_list=[_is_valid_answer])]
        errors = manipulator.get_validation_errors(new_data)
        if not errors :
            manipulator.do_html2python(new_data)
            manipulator.save(new_data)
            return HttpResponseRedirect('%s' % (entry.get_absolute_url(),))
    else :
        errors = new_data = {}
    form = forms.FormWrapper(manipulator, new_data, errors)
    return render_to_response('blog/entry_detail.html', dict(form=form, object=entry), context_instance=RequestContext(request))

def tbping(request, year, month, day, slug) :
    """
    トラックバック追加用ビュー
    """
    if request.method == 'GET' :
        raise Http404

    manipulator = Trackback.AddManipulator()
    entry = _getEntry(year, month, day, slug)
    new_data = request.POST.copy()
    new_data.update({'entry': entry.id})
    errors = manipulator.get_validation_errors(new_data)
    if not errors :
        manipulator.do_html2python(new_data)
        manipulator.save(new_data)
        msg = 'thanx'
        error = ''
    else:
        msg = 'Your request has problems.'
        error = '\n'.join(['%s: %s' % (k,v) for k,v in errors.items()])
    response = HttpResponse(mimetype='text/xml')
    t = loader.get_template('blog/tbping.xml')
    c = Context({
        'error': error,
        'msg': msg,
    })
    response.write(t.render(c))
    return response

from django.contrib.auth.decorators import login_required
def login_required_view_for_test(request):
    return HttpResponse('ok')
login_required_view_for_test = login_required(login_required_view_for_test)

def _is_valid_answer(data, form) :
    if form.get('valid_answer').upper() != data.upper() :
        raise validators.ValidationError, _('Your answer is not correct. Try it again!')

def _getEntry(year, month, day, slug) :
    try:
        d = ddate(*time.strptime(year+month+day, '%Y%m%d')[:3])
    except ValueError:
        raise Http404
    e_list = Entry.public_objects.filter(slug__exact=slug,
                                  pub_date__range=(datetime.combine(d, dtime.min), datetime.combine(d, dtime.max)))
    if len(e_list) == 0:
        raise Http404
    return e_list[0]

